MeshbluSocketIO = require '../../src/meshblu-socket-io'
{EventEmitter} = require 'events'

describe 'MeshbluSocketIO', ->
  beforeEach ->
    @socketIOClient = new EventEmitter
    @SocketIOClient = sinon.spy => @socketIOClient

  describe '->connect', ->
    describe 'when instantiated with node url params', ->
      beforeEach ->
        config = {
          hostname: 'localhost'
          port: 1234
          uuid: 'some-uuid'
          token: 'some-token'
        }

        @sut = new MeshbluSocketIO config, SocketIOClient: @SocketIOClient
        @sut.identity = sinon.stub()
        @sut.connect (@error, @data) =>

      it 'should instantiate a new socket.io client', ->
        expect(@SocketIOClient).to.have.been.calledWithNew

      it 'should have been called with a formated url', ->
        expect(@SocketIOClient).to.have.been.calledWith 'wss://localhost:1234', path: '/socket.io/v2'

      describe 'when the SocketIOClient connects', ->
        beforeEach ->
          @socketIOClient.emit 'connect'

        it 'should call identity', ->
          expect(@sut.identity).to.have.been.calledWith uuid: 'some-uuid', token: 'some-token'

      describe 'when identity yields no error', ->
        beforeEach ->
          @socketIOClient.emit 'connect'
          @sut.identity.yield null, uuid: 'cheese'

        it 'should call the callback with no error and the uuid', ->
          expect(@error).not.to.exist
          expect(@data).to.deep.equal uuid: 'cheese'

      describe 'when the identity yields an error', ->
        beforeEach ->
          @sut.on 'error', (@error) =>
          @socketIOClient.emit 'connect'

          error = new Error 'unauthorized'
          error.status = 403
          @sut.identity.yield error

        it 'should call the callback with an error', ->
          expect(@error).to.be.an.instanceOf Error
          expect(@error.message).to.deep.equal 'unauthorized'
          expect(@error.status).to.deep.equal 403

  describe '->identity', ->
    describe 'when it works', ->
      beforeEach ->
        sinon.stub @socketIOClient, 'emit'
        config = auto_set_online: false
        @sut = new MeshbluSocketIO config, SocketIOClient: @SocketIOClient
        @sut.socketIOClient = @socketIOClient

        @callback = sinon.spy (@error, @data) =>
        @sut.identity uuid: 'youyouid', token: 'tolkin', @callback

      it 'should instantiate a new socket.io client', ->
        expect(@socketIOClient.emit).to.have.been.calledWith 'identity', uuid: 'youyouid', token: 'tolkin'

      describe 'when identity yields an error', ->
        beforeEach ->
          @socketIOClient.emit.yield [{message: 'bad', status: 512}]

        it 'should call the callback with the error', ->
          expect(@callback).to.have.been.called
          expect(@error).to.be.an.instanceOf Error
          expect(@error.message).to.deep.equal 'bad'
          expect(@error.status).to.deep.equal 512

      describe 'when identity yields no error and a device', ->
        beforeEach ->
          @device = {uuid: 'foo'}
          @socketIOClient.emit.yield [null, @device]

        it 'should call the callback with the device', ->
          expect(@callback).to.have.been.called
          expect(@error).not.to.exist
          expect(@data).to.deep.equal @device


  describe '->update', ->
    beforeEach ->
      sinon.spy @socketIOClient, 'emit'
      config =
        uuid: 'youyouid'
        token: 'tolkin'
        auto_set_online: false
      @sut = new MeshbluSocketIO config
      @sut.socketIOClient = @socketIOClient

    describe 'when update is called with a query and params, and the update yields a response with an error', ->
      beforeEach ->
        @socketIOClient.on 'update', (data, callback) => callback([{message: 'error', status: 507}])
        @sut.update {uuid: 'this'}, {pigeonCount: 3}, (@error) =>

      it 'should call the callback with an error', ->
        expect(@error).to.be.an.instanceOf Error
        expect(@error.message).to.deep.equal 'error'
        expect(@error.status).to.deep.equal 507

    describe 'when update is called and everything is OK', ->
      beforeEach ->
        @device = {pigeonCount: 3}
        @socketIOClient.on 'update', (@data, callback) => callback([null])
        @sut.update {uuid: 'this'}, {pigeonCount: 3}, (@error) =>

      it 'should call update on the socketIOClient', ->
        expect(@data).to.deep.equal [{uuid: 'this'}, {$set: {pigeonCount: 3}}]

      it 'should call the callback with a device', ->
        expect(@error).not.to.exist

  describe '->updateDangerously', ->
    beforeEach ->
      sinon.spy @socketIOClient, 'emit'
      config =
        uuid: 'youyouid'
        token: 'tolkin'
        auto_set_online: false
      @sut = new MeshbluSocketIO config
      @sut.socketIOClient = @socketIOClient

    describe 'when update is called with a query and params, and the update yields a response with an error', ->
      beforeEach ->
        @socketIOClient.on 'update', (data, callback) => callback([{message: 'error', status: 507}])
        @sut.updateDangerously {uuid: 'this'}, {pigeonCount: 3}, (@error) =>

      it 'should call the callback with an error', ->
        expect(@error).to.be.an.instanceOf Error
        expect(@error.message).to.deep.equal 'error'
        expect(@error.status).to.deep.equal 507

    describe 'when update is called and everything is OK', ->
      beforeEach ->
        @device = {pigeonCount: 3}
        @socketIOClient.on 'update', (@data, callback) => callback([null])
        @sut.updateDangerously {uuid: 'this'}, {pigeonCount: 3}, (@error) =>

      it 'should call update on the socketIOClient', ->
        expect(@data).to.deep.equal [{uuid: 'this'}, {pigeonCount: 3}]

      it 'should call the callback with a device', ->
        expect(@error).not.to.exist
